#include "libswi2c.h"
#include "libwait.h"

unsigned char i2c_adr0;		// 				<i2c_adr2>	<i2c_adr1>	<i2c_adr0>
unsigned char i2c_adr1;		// 256Kbits:	1010_0AAW	0aaa_aaaa	aaaa_aaaa	15bits/ 32Kbytes
unsigned char i2c_adr2;		//   1Mbits:	1010_AAaW	aaaa_aaaa	aaaa_aaaa	17bits/128Kbytes
unsigned char i2c_ctrl;
unsigned char i2c_data;
unsigned char i2c_data1;
unsigned char i2c_data2;

/*------------------------------------------------------------------------------
 *
 *	I2C MASTER MODE 初期設定
 *
 */
void i2c_init() {
	I2C_SDA = 1;
	I2C_TRISCL = I2C_OUT;
	I2C_SCL = 1;
}

												// 2.7-volt
												// fSCL		400		kHz(2.5us)
												// tLOW		1.3		us
												// tHIGH	0.6		us
												// tAA		0.05	us
												// tBUF		1.3		us
												// tHD.STA	0.6		us
												// tSU.STA	0.6		us
												// tHD.DAT	0		us
												// tSU.DAT	100		ns
												// tR		0.3		us(Max)
												// tF		300		ns(Max)
												// tSU.STO	0.6		us
												// tDH		50		ns
												// tWR		10		ms(Max)

void i2c_start_condition() {

	I2C_SDA = 1;
	I2C_TRISDA = I2C_OUT;
	I2C_SCL = 1;
	NOP();										// tSU.STA	0.6		us
	I2C_SDA = 0;
	NOP();										// tHD.STA	0.6		us
	I2C_SCL = 0;
}

void i2c_transmit(unsigned char c) {

	unsigned char i;

	I2C_TRISDA = I2C_OUT;
	for(i = 0; i < 8; ++i) {
												// tAA		0.05	us
												// tDH		50		ns
		if(c & 0x80) {
			I2C_SDA = 1;
		} else {
			I2C_SDA = 0;
		}
		NOP();									// tLOW		1.3		us
		I2C_SCL = 1;
		c <<= 1;								// tHIGH	0.6		us
		I2C_SCL = 0;
	}
}

void i2c_wait_ack() {

	I2C_TRISDA = I2C_IN;
	NOP();										// tLOW		1.3		us
	I2C_SCL = 1;
	NOP();										// tHIGH	0.6		us
	while(I2C_SDA);								// wait ACK
	I2C_SCL = 0;
}

unsigned char i2c_receive() {

	unsigned char r = 0, i;

	I2C_TRISDA = I2C_IN;
	for(i = 0; i < 8; ++i) {
		NOP();									// tLOW		1.3		us
		I2C_SCL = 1;
		NOP();									// tHIGH	0.6		us
		r <<= 1;
		if(I2C_SDA) r += 1;
		I2C_SCL = 0;
	}
	return(r);
}

void i2c_transmit_ack() {

	I2C_SDA = 0;								// ACK
	I2C_TRISDA = I2C_OUT;
	NOP();										// tLOW		1.3		us
	I2C_SCL = 1;
	NOP();										// tHIGH	0.6		us
	I2C_SCL = 0;
}

void i2c_transmit_nack() {

	I2C_SDA = 1;								// NACK
	I2C_TRISDA = I2C_OUT;
	NOP();										// tLOW		1.3		us
	I2C_SCL = 1;
	NOP();										// tHIGH	0.6		us
	I2C_SCL = 0;
}

void i2c_stop_condition() {

	I2C_SDA = 0;
	I2C_TRISDA = I2C_OUT;
	I2C_SCL = 1;
	NOP();										// tSU.STO	0.6		us
	I2C_SDA = 1;
	NOP();										// tBUF		1.3		us
}

/*------------------------------------------------------------------------------
 *
 *	I2C EEPROM バイト書き込み
 *
 *	i2c_adr2 = I2C_EEPROM_ADR;
 *	i2c_adr1 = 0x01;
 *	i2c_adr0 = 0x80;
 *	i2c_data = 0x55;
 *	i2c_eeprom_byte_write();
 */
void i2c_eeprom_byte_write() {

	i2c_start_condition();						// Start condition

	i2c_transmit((i2c_adr2 << 1) + 0);			// Transmit address(0:write)
	i2c_wait_ack();

	i2c_transmit(i2c_adr1);						// Transmit address
	i2c_wait_ack();

	i2c_transmit(i2c_adr0);						// Transmit address
	i2c_wait_ack();

	i2c_transmit(i2c_data);						// Transmit data
	i2c_wait_ack();

	i2c_stop_condition();						// Stop condition

	waitM(20);									// wait 10ms
}

/*------------------------------------------------------------------------------
 *
 *	I2C EEPROM ページ書き込み
 *
 *	i2c_adr2 = I2C_EEPROM_ADR;
 *	i2c_adr1 = 0x01;
 *	i2c_adr0 = 0x80;
 *	i2c_eeprom_page_write_start();
 *	i2c_data = 0x55;
 *	i2c_eeprom_page_write();
 *	i2c_data = 0xAA;
 *	i2c_eeprom_page_write();
 *	i2c_eeprom_page_write_stop();
 */
void i2c_eeprom_page_write_start() {

	i2c_start_condition();						// Start condition

	i2c_transmit((i2c_adr2 << 1) + 0);			// Transmit address(0:write)
	i2c_wait_ack();

	i2c_transmit(i2c_adr1);						// Transmit address
	i2c_wait_ack();

	i2c_transmit(i2c_adr0);						// Transmit address
	i2c_wait_ack();
}

void i2c_eeprom_page_write() {

	i2c_transmit(i2c_data);						// Transmit data
	i2c_wait_ack();
}

void i2c_eeprom_page_write_stop() {

	i2c_stop_condition();						// Stop condition

	waitM(20);									// wait 10ms
}

/*------------------------------------------------------------------------------
 *
 *	I2C EEPROM ランダム読み込み
 *
 *	i2c_adr2 = I2C_EEPROM_ADR;
 *	i2c_adr1 = 0x01;
 *	i2c_adr0 = 0x80;
 *	i2c_eeprom_random_read();
 *	i2c_eeprom_cur_read();
 *	x = i2c_data;
 */
void i2c_eeprom_random_read() {

	i2c_start_condition();						// Start condition

	i2c_transmit((i2c_adr2 << 1) + 0);			// Transmit address(0:write)
	i2c_wait_ack();

	i2c_transmit(i2c_adr1);						// Transmit address
	i2c_wait_ack();

	i2c_transmit(i2c_adr0);						// Transmit address
	i2c_wait_ack();

	i2c_stop_condition();						// Stop condition
}

/*------------------------------------------------------------------------------
 *
 *	I2C EEPROM 連続読み込み
 *
 *	i2c_adr2 = I2C_EEPROM_ADR;
 *	i2c_eeprom_cur_read();
 *	x = i2c_data;
 */
void i2c_eeprom_cur_read() {

	i2c_start_condition();						// Start condition

	i2c_transmit((i2c_adr2 << 1) + 1);			// Transmit address(1:read)
	i2c_wait_ack();

	i2c_data = i2c_receive();					// Receive data
	i2c_transmit_nack();						// NACK

	i2c_stop_condition();						// Stop condition
}

/*------------------------------------------------------------------------------
 *
 *	I2C 液晶ディスプレイへの送信
 *
 *	i2c_adr0 = I2C_LCD_ADR;
 *	i2c_ctrl = RS_HIGH;
 *	i2c_data = 0x01;
 *	i2c_st7032_send();
 */
void i2c_st7032_send() {

	i2c_start_condition();						// Start condition

	i2c_transmit((i2c_adr0 << 1) + 0);			// Transmit address(0:write)
	i2c_wait_ack();

	i2c_transmit(i2c_ctrl);						// Transmit control byte
	i2c_wait_ack();

	i2c_transmit(i2c_data);						// Transmit data
	i2c_wait_ack();

	i2c_stop_condition();						// Stop condition
}

/*------------------------------------------------------------------------------
 *
 *	I2C 温度センサの読み込み
 *
 *	i2c_adr0 = I2C_SHT21_ADR;
 *	i2c_ctrl = I2C_SHT21_T;
 *	i2c_sht21_nohold();
 *
 *	湿度
 *	01100011 01010010
 *	01100011 010100xx
 *	0x6350 = 25424 
 *	(25424 x 125 / 65536) - 6 = 42.49%
 *
 *	温度
 *	01100011 01010010
 *	01100011 010100xx
 *	0x6350 = 25424 
 *	(25424 x 175.72 / 65536) - 46.85 = 42.49%
 */
void i2c_sht21_nohold() {

	i2c_start_condition();						// Start condition

	i2c_transmit((i2c_adr0 << 1) + 0);			// Transmit address(0:write)
	i2c_wait_ack();

	i2c_transmit(0b11100000 | i2c_ctrl);		// Transmit command
	i2c_wait_ack();

	i2c_stop_condition();						// Stop condition

	waitL(1);									// wait 100ms

	i2c_start_condition();						// Start condition

	i2c_transmit((i2c_adr0 << 1) + 1);			// Transmit address(1:read)
	i2c_wait_ack();

	i2c_data = i2c_receive();					// Receive data
	i2c_transmit_ack();							// ACK

	i2c_data1 = i2c_receive();					// Receive data
	i2c_transmit_ack();							// ACK

	i2c_data2 = i2c_receive();					// Receive data
	i2c_transmit_nack();						// NACK

	i2c_stop_condition();						// Stop condition
}

/*------------------------------------------------------------------------------
 *
 *	I2C 気圧センサの書き込み
 *
 *	i2c_adr0 = I2C_SHT21_ADR;
 *	i2c_ctrl = 0x0F;
 *	i2c_data = 0x0F;
 *	i2c_lps331ap_read();
 */
void i2c_lps331ap_write() {

	i2c_start_condition();						// Start condition

	i2c_transmit((i2c_adr0 << 1) + 0);			// Transmit address(0:write)
	i2c_wait_ack();

	i2c_transmit(i2c_ctrl);						// Transmit register address
	i2c_wait_ack();

	i2c_transmit(i2c_data);						// Transmit command
	i2c_wait_ack();

	i2c_stop_condition();						// Stop condition
}

/*------------------------------------------------------------------------------
 *
 *	I2C 気圧センサの読み込み
 *
 *	i2c_adr0 = I2C_SHT21_ADR;
 *	i2c_ctrl = 0x0F;
 *	i2c_lps331ap_read();
 */
void i2c_lps331ap_read() {

	i2c_start_condition();						// Start condition

	i2c_transmit((i2c_adr0 << 1) + 0);			// Transmit address(0:write)
	i2c_wait_ack();

	i2c_transmit(i2c_ctrl);						// Transmit register address
	i2c_wait_ack();

	i2c_stop_condition();						// Stop condition

	i2c_start_condition();						// Start condition

	i2c_transmit((i2c_adr0 << 1) + 1);			// Transmit address(0:read)
	i2c_wait_ack();

	i2c_data = i2c_receive();					// Receive data
	i2c_transmit_nack();						// NACK

	i2c_stop_condition();						// Stop condition
}

