#include <sdcc-lib.h>
#include <pic16f648a.h>	// /usr/share/sdcc/non-free/include/pic14/pic16f648a.h, /usr/share/sdcc/include/pic14/pic16f648a.h
#include "inline.h"
#include "libwait.h"
#include "libusart.h"
#include "libswi2c.h"
#include "libi2clcd.h"
#include "libxbeeapiframe.h"
#include "libbcd20.h"

static __code char __at (0x2000) id_location[] = {'A', 'B', 'C', 'D'};

static __code int __at (0x2007) configword1 =
	_CP_OFF &
	_DATA_CP_OFF &
	_LVP_OFF &
	_BOREN_OFF &	// for 4.0V
	_MCLRE_OFF &
	_PWRTE_OFF &
	_WDT_ON &
	_INTOSC_OSC_NOCLKOUT;

static __code char __at 0x2100 eedata[] = {'d', 'u', 'm', 'm', 'y', 0};

#define	LED1	RA0
#define	XBSLEEP	RB0

void Intr(void) __interrupt 0 {
	receiveUsartIntr();
}

/*			  _______				PIC16F648A
	NC		-|2	 U	1|-	NC
	NC		-|3		0|-	LED1
	NC		-|4	 A	7|-	NC
	NC		-|5		6|-	NC
	GND		-|		 |-	Vcc
	XBSLEEP	-|0		7|-	NC
	RX		-|1		6|-	NC
	TX		-|2	 B	5|-	NC
	SCL		-|3_____4|-	SDA
*/
/*			  _______				SHT21 温湿度センサ
	VDD		-|V		 |
	SCL		-|C		 |
	SDA		-|D		 |
	VSS		-|G_____ |
*/
/*			  _______				LPS331AP 気圧センサ
	GND		-|1  U 10|-	Vdd_IO
	SCL		-|2		9|-	Vdd
	SDA		-|3		8|-	GND
	SA0		-|4		7|-	INT1
	CS		-|5_____6|-	INT2
*/

void main(void) {

	unsigned int v16, result;		// 16bit
	unsigned char i, r;

	struct xbee_API_Frame_XMIT_REQ		*xbuf_xr = (struct xbee_API_Frame_XMIT_REQ*)xbee_buf_tx;
	struct xbee_API_Frame_MODEM_STAT	*xbuf_md = (struct xbee_API_Frame_MODEM_STAT*)xbee_buf_rx;
	struct xbee_API_Frame_XMIT_STAT		*xbuf_xs = (struct xbee_API_Frame_XMIT_STAT*)xbee_buf_rx;
	struct xbee_API_Frame_RECV_PCKT		*xbuf_rp = (struct xbee_API_Frame_RECV_PCKT*)xbee_buf_rx;

				//  76543210
	PORTA		= 0b00000000;		// RA4: open drain, RA5: input only
	CMCON		= 0b00000111;
	TRISA		= 0b00100000;		// 1: 入力, 0: 出力

	PORTB		= 0b00000000;
	TRISB		= 0b11000110;		// 1: 入力, 0: 出力

	OPTION_REG	= 0b01111111;		// 初期値:0b11111111
									// 7:PORTBプルアップ(0:使用する)
									// 6:RB0割り込みポート変化選択(0:立下り)
									// 5:TMR0クロックソース選択ビット(0:内部)
									// 4:TMR0加算タイミング(0:立上り)
									// 3:プリスケーラ割り当てビット(0:TMR0)
									// 2~0プリスケーラレート選択ビット(011:1/16)
									// 4MHz => 1MHz = 1,000,000 / 16 / 256 = 244Hz

	INTCON		= 0b11000000;		// 初期値:0b0000000x
									// 7:GIE  Global Interrupt Enable bit
									// 6:PEIE Peripheral Interrupt Enable bit
									// 5:T0IE TMR0 Overflow Interrupt Enable bit
									// 4:INTE RB0/INT External Interrupt Enable bit
									// 3:RBIE RB Port Change Interrupt Enable bit
									// 2:T0IF TMR0 Overflow Interrupt Flag bit
									// 1:INTF RB0/INT External Interrupt Flag bit
									// 0:RBIF RB Port Change Interrupt Flag bit

	PIR1		= 0b00000000;		// 初期値:0b00000000(フラグ)
	PIE1		= 0b00000000;		// 初期値:0b00000000(イネーブル)
									// 7:EEIE EE Write Complete Interrupt Enable Bit
									// 6:CMIE Comparator Interrupt Enable bit
									// 5:RCIE USART Receive Interrupt Enable bit
									// 4:TXIE USART Transmit Interrupt Enable bit
									// 2:CCP1IE CCP1 Interrupt Enable bit
									// 1:TMR2IE TMR2 to PR2 Match Interrupt Enable bit
									// 0:TMR1IE TMR1 Overflow Interrupt Enable bit

	T1CON		= 0b00000000;		// 初期値:0b00000000
									// 5~4:T1CKPS Timer1 Input Clock Prescale Select bits(00:1/1, 11:1/8)
									// 3:T1OSCEN Timer1 Oscillator Enable Control bit(1:Enable)
									// 2:T1SYNC Timer1 External Clock Input Synchronization Control bit(1:Sync)
									// 1:TMR1CS Timer1 Clock Source Select bit(0:Internal)
									// 0:TMR1ON Timer1 On bit(1:Enable)

	T2CON		= 0b00000000;		// 初期値:0b00000000
									// 6~3:TOUTPS Timer2 Output Postscale Select bits(0000:1/1, 0011:1/4, 1111:1/16)
									// 2:TMR2ON Timer2 On bit(1:ON)
									// 1~0:T2CKPS Timer2 Clock Prescale Select bits(00:1/1, 01:1/4, 1x:1/16)

//	forCalibOsc();
	initUsart();
	xbee_API_Frame_init();
	i2c_init();

	//--------------------------------------------------------------------------
	//
	//	Main Loop
	//
	LED1 = 1;
	waitL(10);
	LED1 = 0;

	lcd_init();													// LCD 表示テスト
	lcd_clear();
	lcd_title();

//	LCD 表示イメージ
//	----++++----++++
//	12/31 12:34 D
//	25.0c 60% 1013mb

	i2c_adr0 = I2C_LPS331AP_ADR1;								// 気圧センサ初期化
	i2c_ctrl = I2C_LPS331AP_CTRL_REG1;
	i2c_data = 0b10010000;										// Power: active / Rate: 1Hz
	i2c_lps331ap_write();										// TODO: 消費電力を下げる(ONESHOT?)

	CLEAR_WDT();
	i = 0;

	while(1) {

		if(++i % 20 != 0) {										// WDT 周期: 約 21 ms(3.3V/25C) x 128 = 2.7 sec
			CLEAR_WDT();
			SLEEP();

		} else {												// 動作周期: 約 2.7 sec x 20 = 54 sec
			XBSLEEP = 0;
			LED1 = 1;
			waitM(4);											// 4: 2ms (経験的設定, 1.5ms でも OK)

			xbuf_xr->id++; xbuf_xr->id |= 0x80;
			xbee_buf_txlen = 12;
			xbee_API_Frame_begin();
			xbee_API_Frame_putc(0x11);							// 情報種別コード

			//--------------------------------------------------
			//
			//	温度
			//
			i2c_adr0 = I2C_SHT21_ADR;							// 温度を取得(温湿度センサ)
			i2c_ctrl = 0b11110000 | I2C_SHT21_T;
			i2c_sht21_nohold();

			xbee_API_Frame_putc(i2c_data);						// 温度を送信
			xbee_API_Frame_putc(i2c_data1);						// ([i2c_data:i2c_data1] x 175.72 / 65536) - 46.85 (℃)
			xbee_API_Frame_putc(i2c_data2);

			v16 = (i2c_data << 8) + i2c_data1;					// 温度を変換
			result = (v16 / 119) << 5;
			result += ((v16 % 119) << 8) / 952;
			result -= 4705;

			b20bcd2 = 0;										// 温度をBCD変換
			b20bcd1 = result >> 8;
			b20bcd0 = result & 0xFF;
			bin20_to_bcd();

			lcd_locate(0, 1);									// 温度を表示
			lcd_putc('0' + (b20bcd4 & 0x0F));
			lcd_putc('0' + (b20bcd3 >> 4));
			lcd_putc('.');
			lcd_putc('0' + (b20bcd3 & 0x0F));
			lcd_putc(7);										// ℃

			//--------------------------------------------------
			//
			//	湿度
			//
			i2c_adr0 = I2C_SHT21_ADR;							// 湿度を取得(温湿度センサ)
			i2c_ctrl = 0b11110000 | I2C_SHT21_RH;
			i2c_sht21_nohold();

			xbee_API_Frame_putc(i2c_data);						// 湿度を送信
			xbee_API_Frame_putc(i2c_data1);						// ([i2c_data:i2c_data1] x 125 / 65536) - 6 (%)
			xbee_API_Frame_putc(i2c_data2);

			v16 = (i2c_data << 8) + i2c_data1;					// 湿度を変換
			result = (v16 / 168) << 5;
			result += ((v16 % 168) << 8) / 1344;
			result -= 592;

			b20bcd2 = 0;										// 湿度をBCD変換
			b20bcd1 = result >> 8;
			b20bcd0 = result & 0xFF;
			bin20_to_bcd();

			lcd_putc(' ');										// 温度を表示
			lcd_putc('0' + (b20bcd4 & 0x0F));
			lcd_putc('0' + (b20bcd3 >> 4));
			lcd_putc('%');

			//--------------------------------------------------
			//
			//	気圧(温度)
			//
			i2c_adr0 = I2C_LPS331AP_ADR1;						// 気圧(温度)を取得(気圧センサ)
			i2c_ctrl = I2C_LPS331AP_PRESS_OUT_H;
			i2c_lps331ap_read();
			xbee_API_Frame_putc(i2c_data);
			v16 = i2c_data << 8;								// 'HH00'

			i2c_adr0 = I2C_LPS331AP_ADR1;
			i2c_ctrl = I2C_LPS331AP_PRESS_OUT_L;
			i2c_lps331ap_read();
			xbee_API_Frame_putc(i2c_data);
			v16 += i2c_data;									// 'HHLL'
			result = v16 >> 4;									// '0HHL'

			i2c_adr0 = I2C_LPS331AP_ADR1;
			i2c_ctrl = I2C_LPS331AP_PRESS_OUT_XL;				// [H:L:XL] / 4096 (mbar/hPa)
			i2c_lps331ap_read();								// 標準大気圧は海面上で 1013.25 hPa
			xbee_API_Frame_putc(i2c_data);						// -1 hPa => +10m, -0.01 hPa => +10cm 

			b20bcd2 = 0;										// 気圧をBCD変換
			b20bcd1 = result >> 8;
			b20bcd0 = result & 0xFF;
			bin20_to_bcd();

			lcd_putc(' ');										// 気圧を表示
			lcd_putc('0' + (b20bcd4 & 0x0F));
			lcd_putc('0' + (b20bcd3 >> 4));
			lcd_putc('0' + (b20bcd3 & 0x0F));
			lcd_putc('0' + (b20bcd2 >> 4));
			lcd_putc('m');
			lcd_putc('b');

			i2c_adr0 = I2C_LPS331AP_ADR1;
			i2c_ctrl = I2C_LPS331AP_TEMP_OUT_H;
			i2c_lps331ap_read();
			xbee_API_Frame_putc(i2c_data);

			i2c_adr0 = I2C_LPS331AP_ADR1;
			i2c_ctrl = I2C_LPS331AP_TEMP_OUT_L;					// [(signed)H:L] / 480 (℃)
			i2c_lps331ap_read();
			xbee_API_Frame_putc(i2c_data);

			xbee_API_Frame_end();

			waitM(200);											// 相手の反応を少し待つ

			while(1) {
				while((r = xbee_API_Frame_receive_rb()) == 1);	// フレーム受信 / 1: フレーム受信中

				if(r == 0) {									// 0: フレーム受信成功
					switch(xbuf_rp->type) {
					  case XBEE_RECV_PCKT:
						if(xbuf_rp->dest64[7] == 0xE3) {
							lcd_locate(0, 0);
							lcd_putc('0' + (xbuf_rp->data[0] >> 4));
							lcd_putc('0' + (xbuf_rp->data[0] & 0x0F));
							lcd_putc('/');
							lcd_putc('0' + (xbuf_rp->data[1] >> 4));
							lcd_putc('0' + (xbuf_rp->data[1] & 0x0F));
							lcd_putc(' ');
							lcd_putc(xbuf_rp->data[3] % 7);		// TODO: 暫定
							lcd_putc(' ');
							lcd_putc('0' + (xbuf_rp->data[2] >> 4));
							lcd_putc('0' + (xbuf_rp->data[2] & 0x0F));
							lcd_putc(':');
							lcd_putc('0' + (xbuf_rp->data[3] >> 4));
							lcd_putc('0' + (xbuf_rp->data[3] & 0x0F));
						}
						break;
					}
				} else {										// 2: チェックサムエラー / 3: タイムアウト
					break;
				}
			}

			waitM(200);											// 200: 100ms (経験的設定)
			LED1 = 0;
			XBSLEEP = 1;
		}
	}
}

