#!/usr/bin/env ruby

# print current settings
#   $ stty -g < /dev/ttyUSB2

$config = Hash.new
$config['tty']  = '/dev/ttyUSB2'
#config['stty'] = '1:0:800008bb:0:3:1c:7f:15:4:5:1:0:11:13:1a:0:12:f:17:16:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0'	#  2400bps
#config['stty'] = '1:0:800008bd:0:3:1c:7f:15:4:5:1:0:11:13:1a:0:12:f:17:16:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0'	#  9600bps
$config['stty'] = '1:0:8bd:0:3:1c:7f:15:4:5:1:0:11:13:1a:0:12:f:17:16:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0'		# -crtscts
#config['stty'] = '1:0:800018b1:0:3:1c:7f:15:4:5:1:0:11:13:1a:0:12:f:17:16:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0'	# 57600bps

class IO
	def getc
		RUBY_VERSION < '1.9' ? super : self.read(1).ord
	end
end

class XBee_API_Frame

	def add_len_sum

		len = @frame.size - 3
		@frame[1] = len >> 8
		@frame[2] = len & 0xFF
		sum = 0; (3...@frame.size).each {|p|
			sum = (sum + @frame[p]) & 0xFF
		}
		@frame << (0xFF - sum)
	end

	def transmit_request(data, id = 1, dest = 0x000000000000FFFF, dest16 = 0xFFFE)

		@frame = [ 0x7E, 0x00, 0x00, 0x10 ]						# Frame Type
		@frame << id
		adr = []; 8.times {
			adr.unshift(dest & 0xFF); dest >>= 8
		}
		@frame += adr
		@frame << (dest16 >> 8)
		@frame << (dest16 & 0xFF)
		@frame << 0x00											# Broadcast Radius
		@frame << 0x00											# Options
		@frame += data.unpack('c*')
		add_len_sum
	end

	def receive_request(c)

		if(@xbuf_rx)											# フレーム受信中
			@xbuf_rx << c
			@xbuf_rxsum = (@xbuf_rxsum + c) & 0xFF

			if(@xbuf_rx.size == 3)								# Length まで受信したらチェックサム加算を開始
				@xbuf_rxlen = (@xbuf_rx[1] << 8) + @xbuf_rx[2]
				@xbuf_rxsum = 0

			elsif(@xbuf_rx.size == @xbuf_rxlen + 4)				# チェックサムまで受信
				@frame = @xbuf_rx; @xbuf_rx = nil				# フレーム受信完了
				return(@xbuf_rxsum == 0xFF ? @frame : false)
			end

		elsif(c == 0x7E)										# Start Delimiter 検出
			@xbuf_rx = [c]										# フレーム受信開始
			@xbuf_rxlen = @xbuf_rxsum = 0xFF					# Length, チェックサム仮設定
		end

		nil
	end

	def show_frame(frame)

		begin
			require '/usr/local/bin/ndump.rb'
			dump = Ndump16headless.new
			frame.each {|c|
				print dump << c
			}
			print dump.tail

		rescue LoadError
			frame.each {|c|
				print '%02X ' % c
			}
			puts
		end
	end
end

xbee = XBee_API_Frame.new

require 'socket'
target = TCPSocket.open('172.24.0.1', 10001)					# to XPort

#system("stty #{$config['stty']} < #{$config['tty']}")

#open($config['tty'], 'w+') {|target|
	id = 0
	loop {
		if(IO.select([target], [], [], 3))
			c = target.getc
			frame = xbee.receive_request(c)
			if(frame)
				puts
				xbee.show_frame(frame)

				if(frame[3] == 0x90)
					open('tempsat.dat', 'a') {|fh|
						(15..26).each {|c|
							fh.putc(frame[c])
						}
						4.times {
							fh.putc(0)
						}
					}
				end

				# 時刻情報送信
				message = []; now = Time.now
				message << (now.month / 10 << 4) + now.month % 10
				message << (now.day   / 10 << 4) + now.day   % 10
				message << (now.hour  / 10 << 4) + now.hour  % 10
				message << (now.min   / 10 << 4) + now.min   % 10
				frame = xbee.transmit_request(message.pack('c*'), id = (id + 1) % 128 + 128)

				xbee.show_frame(frame)
				frame.each {|c|
					target.putc(c)
				}
			else
				print '>'
				print c
				next
			end
		else
			puts; puts 'timeout'
		end
	}
#}

__END__

